﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.Linq;
using System.Text.RegularExpressions;
using System.Text;
using System.Threading.Tasks;
using System.Timers;
using System.IO;

namespace Lame
{
    class Lame
    {
        private const int LAME_TIMER_STATUS_UPDATE_INTERVALL = 1000;
        private String[] LameStaticParameters = { "--nohist", "--disptime 1 --flush" };
        private String LameParameterTemplate = "{0} {1} -b {2} --ta \"{3}\" --tt \"{4}\" -b {5} \"{6}\" \"{7}\"";
        private String LameParameter;
        private FileInfo oLameExe;
        public Song.Song oSong;
        // Timer LameStateTimer;
        Process oLameProcess;
        LameOptions Options;
        Regex oStatefromSTDOUT = new Regex(@"\((\d+)%\).+\|.+\|.+\|.+$");
        private int iStateOfConvert=0;
        public class Bitrate
        {
            public Bitrate(int Value)
            {
                this.Value = Value;
            }
            public int Value { get; set; }
            public String DisplayName { get { return (this.Value + " kbps"); } }
        }
        public static List<Bitrate> Bitrates(String Bitrates)
        {
            List<Bitrate> BitrateList=new List<Bitrate>();
            String[] aBitrates = Bitrates.Split(new Char[] {';'},StringSplitOptions.RemoveEmptyEntries);
            foreach (String Bitrate in aBitrates)
            { 
                int Bits=0;
                if(int.TryParse(Bitrate, out Bits ))
                    BitrateList.Add(new Bitrate(Bits));
            }
            return BitrateList;
        }
        public class LameOptions
        {
            public String outputfile { get; set; }
            public String inputfile { get; set; }
            public int bitrateInKByte { get; set; }
            public String Artist { get; set; }
            public String Songtitle { get; set; }
            public Song.Song Song { get; set; }
            public bool DeleteSourceFile { get; set; }
            public String LameCustomParameters { get; set; }
            public LameOptions(String Artistname, String Songtitle, int Bitrate, String InputFile)
            {
                this.inputfile = InputFile;
                this.bitrateInKByte = Bitrate;
                this.Artist = Artistname;
                this.Songtitle = Songtitle;
                Song = new Song.Song(Artistname, Songtitle, Path.GetDirectoryName(InputFile));
                this.outputfile = Song.MP3FilePath;
                DeleteSourceFile = true;
            }
            public LameOptions(Song.Song Song, int Bitrate)
            {
                this.outputfile = Song.MP3FilePath;
                this.inputfile = Song.WavFilePath;
                this.bitrateInKByte = Bitrate;
                this.Artist = Song.Artist;
                this.Songtitle = Song.Title;
                this.Song = Song;
                DeleteSourceFile = true;
            }
        }
        public Lame(FileInfo AppStartPath, LameOptions Options)
        {
            this.Options = Options;
            oLameExe = new FileInfo(AppStartPath.DirectoryName + @"\Lame.exe");
            if (!oLameExe.Exists)
                throw new FileNotFoundException("Lame.exe not found in application directory " + AppStartPath.DirectoryName);
            if (!File.Exists(Options.inputfile))
                Options.Song.CoversationState = (int)Song.Song.CONVERSATION_STATE.CONVERT_ERROR;
            oSong = Options.Song;
            LameParameter = String.Format(LameParameterTemplate, String.Join(" ", LameStaticParameters),Options.LameCustomParameters, Options.bitrateInKByte, Options.Artist, Options.Songtitle, Options.bitrateInKByte, Options.inputfile, Options.outputfile);
            if(!String.IsNullOrEmpty(Options.Song.Album))
                LameParameter += @" --tl """+Options.Song.Album+@"""";
            if (!String.IsNullOrEmpty(Options.Song.AlbumImagePath) && System.IO.File.Exists(Options.Song.AlbumImagePath))
                LameParameter += @" --ti """ + Options.Song.AlbumImagePath + @"""";
        }
        public void Convert2MP3()
        {
            oLameProcess = new Process();
            oLameProcess.StartInfo.UseShellExecute = false;
            oLameProcess.StartInfo.RedirectStandardOutput = true;
            oLameProcess.StartInfo.RedirectStandardError = true;
            oLameProcess.StartInfo.FileName = oLameExe.FullName;
            oLameProcess.StartInfo.Arguments = LameParameter;
            oLameProcess.StartInfo.CreateNoWindow=true;
            oLameProcess.EnableRaisingEvents = true;
            oLameProcess.Exited += OnLameExit;
            System.Diagnostics.Debug.WriteLine("Starting Lame.\n\tExe: " + oLameProcess.StartInfo.FileName + "\n\tArguments:" + oLameProcess.StartInfo.Arguments);
            oLameProcess.ErrorDataReceived += new DataReceivedEventHandler(OnDataErrorReceived);
            oLameProcess.Start();
            oLameProcess.BeginErrorReadLine();
        }
        private void OnDataErrorReceived(object oSendingProcess, DataReceivedEventArgs oStdErrData)
        {
            String sState="0";
            if (!String.IsNullOrEmpty(oStdErrData.Data))
            {
                System.Diagnostics.Debug.WriteLine("Lame stdout: " + oStdErrData.Data);
                Match oStateMatch = oStatefromSTDOUT.Match(oStdErrData.Data);
                if (oStateMatch.Success)
                {
                    sState = oStateMatch.Groups[1].Value;
                    System.Diagnostics.Debug.WriteLine("Lame converting progress: " + sState);
                    try
                    {
                        iStateOfConvert = System.Convert.ToInt16(sState);
                    }
                    catch(Exception e)
                    {
                        // Nothing
                    }
                }
                else
                {
                }
            }
            ConvertingStatsEventArgs oEventArgs = new ConvertingStatsEventArgs();
            oEventArgs.Inputfile=Options.inputfile;
            oEventArgs.Outputfile = Options.outputfile;
            oEventArgs.Songtitle = Options.Songtitle;
            oEventArgs.Artist = Options.Artist;
            oEventArgs.State = iStateOfConvert;
            oEventArgs.Song = oSong;
            oEventArgs.Song.CoversationState = iStateOfConvert;
            StateUpdate(oEventArgs);
        }
        private void OnLameExit(object sender, EventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("Lame exit. Return code: " + oLameProcess.ExitCode.ToString());
            ConvertingFinishedEventArgs oEventArgs=new ConvertingFinishedEventArgs();
            oEventArgs.Inputfile = this.Options.inputfile;
            oEventArgs.Outputfile = this.Options.outputfile;
            oEventArgs.Songtitle = this.Options.Songtitle;
            oEventArgs.Artist = this.Options.Artist;
            oEventArgs.Exitcode=oLameProcess.ExitCode;
            oEventArgs.Song = this.oSong;
            oEventArgs.DeleteSourceFile = this.Options.DeleteSourceFile;
            oEventArgs.Song.LameReturnCode = oLameProcess.ExitCode;
            ConvertingFinshed(oEventArgs);
        }
        protected virtual void ConvertingFinshed(ConvertingFinishedEventArgs e)
        {
            EventHandler<ConvertingFinishedEventArgs> handler = OnRecordingFinshed;
            if (handler != null)
            {
                handler(this, e);
            }
        }
        public event EventHandler<ConvertingFinishedEventArgs> OnRecordingFinshed;
        public class ConvertingFinishedEventArgs : EventArgs
        {
            public Song.Song Song { get; set; }
            public String Inputfile { get; set; }
            public String Outputfile { get; set; }
            public String Songtitle { get; set; }
            public String Artist { get; set; }
            public int Exitcode { get; set; }
            public bool DeleteSourceFile { get; set; }
        }
        protected virtual void StateUpdate(ConvertingStatsEventArgs e)
        {
            EventHandler<ConvertingStatsEventArgs> handler = OnStateUpdate;
            if (handler != null)
            {
                handler(this, e);
            }
        }
        public event EventHandler<ConvertingStatsEventArgs> OnStateUpdate;
        public class ConvertingStatsEventArgs : EventArgs
        {
            public Song.Song Song { get; set; }
            public String Inputfile { get; set; }
            public String Outputfile { get; set; }
            public String Songtitle { get; set; }
            public String Artist { get; set; }
            public int State { get; set; }
        }
    }
}