﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using NAudio.Wave;

namespace Recorder
{
    class Recorder: IDisposable
    {
        WasapiLoopbackCapture CaptureInput;
        WaveFileWriter WaveWriter;
        public bool IsRecording = false;
        public String Band { get; set; }
        public String Songtitle { get; set; }
        public String WavFilePath { get { return oSong.WavFilePath; } }
        //private String sWAVFilePath = "";
        private Song.Song oSong;
        public Recorder(Song.Song Song)
        {
            oSong = Song;
            this.Band = oSong.Artist;
            this.Songtitle = oSong.Title;
            // sWAVFilePath = oSong.WavFilePath;
            init();
        }
        private void init()
        {

                CaptureInput = new WasapiLoopbackCapture();
                WaveFormat saveWaveFormat = CaptureInput.WaveFormat;
                WaveWriter = new WaveFileWriter(oSong.WavFilePath, CaptureInput.WaveFormat);
                CaptureInput.DataAvailable += OnDataAvailable;
        }
        public void StartRecording()
        {
            IsRecording = false;
            if (CaptureInput == null || WaveWriter == null)
                    init();
            try
            {
                CaptureInput.StartRecording();
                IsRecording = true;
                System.Diagnostics.Debug.WriteLine("Recording started.\n\tFile=" + oSong.WavFilePath + "\n\tBand=" + this.Band + "\n\tSong=" + this.Songtitle);
            }
            catch (Exception e)
            {
                System.Diagnostics.Debug.WriteLine("Error occured while start Recording:" + e.Message);
            }
        }
        public void StopRecording()
        {
            if (CaptureInput != null)
            {
                CaptureInput.DataAvailable -= OnDataAvailable;
                CaptureInput.StopRecording();
                GC.SuppressFinalize(CaptureInput);
                CaptureInput.Dispose();
                CaptureInput = null;
            }
            if (WaveWriter != null)
            {
                WaveWriter.Flush();
                WaveWriter.Close();
                WaveWriter.Dispose();
                GC.SuppressFinalize(WaveWriter);
                System.Diagnostics.Debug.WriteLine("Recording finished.\n\tFile=" + oSong.WavFilePath + "\n\tBand=" + this.Band + "\n\tSong=" + this.Songtitle);
                oSong.CoversationState = (int)Song.Song.CONVERSATION_STATE.RECORDING_FINSIHED;
                WaveWriter = null;
            }
            IsRecording = false;
            System.Diagnostics.Debug.WriteLine("Recording of " + oSong.WavFilePath + " stopped");
            FireFinishEvent();
        }
        private void FireFinishEvent()
        {
            RecordingFinishedEventArgs oEventArgsRecFinished = new RecordingFinishedEventArgs();
            oEventArgsRecFinished.Song = oSong;
            oEventArgsRecFinished.Band = this.Band;
            oEventArgsRecFinished.Songtitle = this.Songtitle;
            oEventArgsRecFinished.FilePath = oSong.WavFilePath;
            RecordingFinshed(oEventArgsRecFinished);
        }
        public void Dispose()
        {
            StopRecording();
            GC.SuppressFinalize(this);
        }
        void OnDataAvailable(object sender, WaveInEventArgs e)
        {
            if (WaveWriter != null)
                WaveWriter.Write(e.Buffer, 0, e.BytesRecorded);
        }
        protected virtual void RecordingFinshed(RecordingFinishedEventArgs e)
        {
            EventHandler<RecordingFinishedEventArgs> handler = OnRecordingFinshed;
            if (handler != null)
            {
                handler(this, e);
            }
        }
        public event EventHandler<RecordingFinishedEventArgs> OnRecordingFinshed;
        public class RecordingFinishedEventArgs : EventArgs
        {
            public Song.Song Song { get; set; }
            public String Band { get; set; }
            public String Songtitle { get; set; }
            public String FilePath { get; set; }
        }

        ~Recorder()
        {
            Dispose();
        }
    }
}
