﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
// using CoreAudio;
using NAudio.CoreAudioApi;
using StringLoader;
namespace Sound
{
    class Sound : IDisposable
    {
        public class AudioSession : IDisposable
        {
            // public AudioSessionControl SessionControl { get; set; }
            public String SessionInstanceIdentifier { get; set; }
            public bool isMuted { get; set; }
            public AudioSessionControl Session { get; set; }
            public String SessionName
            {
                get
                {
                    return sSessionName;
                }
            }
            public int Volume {
                get {

                    return (int)(this.Session.SimpleAudioVolume.Volume * 100);
                }
                set {
                    this.Session.SimpleAudioVolume.Volume = (float)(value / 100);
                }
            }
            private String sSessionName = "";
            public EventClient eventClient;
            public AudioSession(AudioSessionControl Session)
            {
                EventClient eventClient = new EventClient();
                eventClient.VolumeChanged += OnVolumeChanged;
                eventClient.StateChanged += OnStateChanged;
                eventClient.SessionDisconnect += OnSessionDisconnect;
                this.SessionInstanceIdentifier = Session.GetSessionInstanceIdentifier;
                this.eventClient = eventClient;
                this.sSessionName = Session.DisplayName;
                this.Session = Session;
                Session.RegisterEventClient(eventClient);
                // Refresh if DLL Path
                this.sSessionName = LoadStringFromDLL.LoadStringFromDLLByPath(Session.DisplayName);
                if (String.IsNullOrEmpty(this.sSessionName))
                {
                    this.sSessionName = GetNameByID(Session.GetProcessID);
                }
                this.isMuted = Session.SimpleAudioVolume.Mute;
                // System.Diagnostics.Debug.WriteLine("New Audio Session: " + sSessionName + " PID:" + Session.GetProcessID.ToString() + " IsMuted:" + this.isMuted);
            }

            public void OnStateChanged(object sender, StateChangedEventArgs e)
            {
                OnStateChanged(e);
            }
            public event EventHandler<StateChangedEventArgs> StateChanged;
            protected virtual void OnStateChanged(StateChangedEventArgs e)
            {
                EventHandler<StateChangedEventArgs> handler = StateChanged;
                if (handler != null)
                {
                    handler(this, e);
                }
            }
            public void OnVolumeChanged(object sender, VolumeEventArgs e)
            {
                this.isMuted = this.Session.SimpleAudioVolume.Mute;
                OnSessionVolumeChanged(e);
            }
            public event EventHandler<VolumeEventArgs> SessionVolumeChanged;
            protected virtual void OnSessionVolumeChanged(VolumeEventArgs e)
            {
                EventHandler<VolumeEventArgs> handler = SessionVolumeChanged;
                if (handler != null)
                {
                    handler(this, e);
                }
            }
            public void OnSessionDisconnect(object sender, DisconnectSessionEventArgs e)
            {
                System.Diagnostics.Debug.WriteLine("SESSION DISCONNECTED: " + e.DisconnectReason.ToString());
                OnSessionDisconnect(e);
            }
            public event EventHandler<DisconnectSessionEventArgs> SessionDisconnect;
            protected virtual void OnSessionDisconnect(DisconnectSessionEventArgs e)
            {
                EventHandler<DisconnectSessionEventArgs> handler = SessionDisconnect;
                if (handler != null)
                {
                    handler(this, e);
                }
            }
            public void Dispose()
            {
                GC.SuppressFinalize(this);
            }
            ~AudioSession()
            {
                // Dispose();
            }
        }
        private MMDeviceEnumerator aMMDevices;
        public MMDeviceEnumerator MMDevices { 
            get{
                return aMMDevices;
            } 
        }
        public MMDevice DefaultAudioEndPointDevice
        {
            get;
            set;
        }
        private AudioSessionManager oSessionDefaultAudioEndPointDevice;
        public AudioSessionManager SessionDefaultAudioEndPointDevice{
            get
            {
                return oSessionDefaultAudioEndPointDevice;
            }
        }
        private SessionCollection coSessionsDefaultAudioEndPointDevice;
        public SessionCollection SessionsDefaultAudioEndPointDevice{
            get
            {
                return coSessionsDefaultAudioEndPointDevice;
            }
        }
        public String SessionInstanceIdentifierSpotify { get; set; }
        public String SpotifyProcessName { get; set; }
        public AudioSessionControl SpodifyAudioSession { get; set; }
        public List<AudioSession> AudioSessions { get; set; } 
        // Thanks to http://stackoverflow.com/questions/14306048/controling-volume-mixer
        // http://whenimbored.xfx.net/2011/01/core-audio-for-net/
        public Sound(){
            SpotifyProcessName = "";
            AudioSessions = new List<AudioSession>();
            aMMDevices = new MMDeviceEnumerator();
            DefaultAudioEndPointDevice = aMMDevices.GetDefaultAudioEndpoint(DataFlow.Render, Role.Multimedia);
            DefaultAudioEndPointDevice.AudioEndpointVolume.OnVolumeNotification += new NAudio.CoreAudioApi.AudioEndpointVolumeNotificationDelegate(AudioEndpointVolumeDelegate);
            oSessionDefaultAudioEndPointDevice = DefaultAudioEndPointDevice.AudioSessionManager;
            coSessionsDefaultAudioEndPointDevice = oSessionDefaultAudioEndPointDevice.Sessions;
            oSessionDefaultAudioEndPointDevice.OnSessionCreated += new AudioSessionManager.SessionCreatedDelegate(AudioSessionCreatedDelegate);
            DefaultAudioDeviceVolume = (int)(DefaultAudioEndPointDevice.AudioEndpointVolume.MasterVolumeLevelScalar * 100);
            LoadAudioSessions();
        }
        public Sound(String SpotifyProcessName)
        {
            this.SpotifyProcessName = SpotifyProcessName;
            AudioSessions = new List<AudioSession>();
            aMMDevices = new MMDeviceEnumerator();
            DefaultAudioEndPointDevice = aMMDevices.GetDefaultAudioEndpoint(DataFlow.Render, Role.Multimedia);
            DefaultAudioEndPointDevice.AudioEndpointVolume.OnVolumeNotification += new NAudio.CoreAudioApi.AudioEndpointVolumeNotificationDelegate(AudioEndpointVolumeDelegate);
            oSessionDefaultAudioEndPointDevice = DefaultAudioEndPointDevice.AudioSessionManager;
            coSessionsDefaultAudioEndPointDevice = oSessionDefaultAudioEndPointDevice.Sessions;
            oSessionDefaultAudioEndPointDevice.OnSessionCreated += new AudioSessionManager.SessionCreatedDelegate(AudioSessionCreatedDelegate);
            DefaultAudioDeviceVolume = (int)(DefaultAudioEndPointDevice.AudioEndpointVolume.MasterVolumeLevelScalar * 100);
            LoadAudioSessions();
        }
        void AudioEndpointVolumeDelegate( NAudio.CoreAudioApi.AudioVolumeNotificationData data)
        {
            // System.Diagnostics.Debug.WriteLine("Master Volume changed" );
            VolumeEventArgs oVolEvntArgs=new VolumeEventArgs(data.MasterVolume,data.Muted);
            DefaultAudioDeviceVolume = (int)(data.MasterVolume * 100);
            OnMasterVolumeChangedOccured(oVolEvntArgs);
        }
        void AudioSessionCreatedDelegate(object sender, NAudio.CoreAudioApi.Interfaces.IAudioSessionControl data)
        {
            System.Diagnostics.Debug.WriteLine("Session created");
            LoadAudioSessions();
            OnNewAudioSessionOccured(new SessionCreatedEventArgs(data));
        }
        public void LoadAudioSessions()
        {
            // Get all current sessions
            System.Collections.Hashtable MissingInstanceIdentifier = new System.Collections.Hashtable();
            foreach (AudioSession Current in AudioSessions)
            {
                MissingInstanceIdentifier.Add(Current.SessionInstanceIdentifier, true);
            }
            try
            {
                oSessionDefaultAudioEndPointDevice.RefreshSessions();
            }
            catch(Exception e)
            {
                System.Diagnostics.Debug.WriteLine("Cannot refresh Audio Sessions." + e.Message);
            }
            coSessionsDefaultAudioEndPointDevice = oSessionDefaultAudioEndPointDevice.Sessions;
            for (int iSession = 0; iSession < coSessionsDefaultAudioEndPointDevice.Count; iSession++)
            {
                if (MissingInstanceIdentifier.ContainsKey(coSessionsDefaultAudioEndPointDevice[iSession].GetSessionInstanceIdentifier))
                {
                    MissingInstanceIdentifier.Remove(coSessionsDefaultAudioEndPointDevice[iSession].GetSessionInstanceIdentifier);
                }
                // System.Diagnostics.Debug.WriteLine("Session:" + coSessionsDefaultAudioEndPointDevice[iSession].GetSessionInstanceIdentifier);
                if (Process.GetCurrentProcess().Id != coSessionsDefaultAudioEndPointDevice[iSession].GetProcessID)
                {
                    AudioSession FindSessions=AudioSessions.Find(s=>s.SessionInstanceIdentifier==coSessionsDefaultAudioEndPointDevice[iSession].GetSessionInstanceIdentifier);
                    if(FindSessions==null)
                    {
                        AudioSession CurrentSession = new AudioSession(coSessionsDefaultAudioEndPointDevice[iSession]);
                        AudioSessions.Add(CurrentSession);
                        CurrentSession.SessionVolumeChanged += OnAllVolumeChanged;
                        CurrentSession.StateChanged += OnAllStateChanged;
                        coSessionsDefaultAudioEndPointDevice[iSession].RegisterEventClient(CurrentSession.eventClient);
                        System.Diagnostics.Debug.WriteLine("Add Audio session:" + coSessionsDefaultAudioEndPointDevice[iSession].GetSessionInstanceIdentifier);
                    }
                }
                Process[] aProcesses = Process.GetProcessesByName(SpotifyProcessName);
                int ProcessCount = aProcesses.Count(s => s.Id == (coSessionsDefaultAudioEndPointDevice[iSession].GetProcessID));
                if (ProcessCount >= 1)
                {
                    // Spotify detected
                    SpodifyAudioSession = coSessionsDefaultAudioEndPointDevice[iSession];
                    SessionInstanceIdentifierSpotify = coSessionsDefaultAudioEndPointDevice[iSession].GetSessionInstanceIdentifier;
                }
            }
            // Remove Left Sessions
            foreach (String OldSession in MissingInstanceIdentifier.Keys)
            {
                AudioSessions.Remove(AudioSessions.Find(s=>s.SessionInstanceIdentifier==OldSession));
            }
            AudioSessionsReload(new EventArgs());
        }
        public int SpodifyVolume
        {
            get
            {
                if(!(String.IsNullOrEmpty(SessionInstanceIdentifierSpotify)))
                {
                    AudioSession ASession = AudioSessions.Find(s => s.SessionInstanceIdentifier == SessionInstanceIdentifierSpotify);
                    if(ASession!=null)
                    {
                        try
                        {
                            return ASession.Volume;
                        }
                        catch
                        {
                            return -1;
                        }

                    }
                    else
                    {
                        return -1;
                    }
                }
                else
                {
                    return -1;
                }
            }
        }
        public void SetSpodifyVolume(int Volume)
        {
            if (SpodifyAudioSession != null)
            {
                try
                {
                    SpodifyAudioSession.SimpleAudioVolume.Volume = ((float)Volume / 100);
                }
                catch
                {
                    System.Diagnostics.Debug.WriteLine("Could not set Volume on Spotify Audiosession");
                }
            }
        }
        public int DefaultAudioDeviceVolume { get; set; }
        public void SetDefaultAudioDeviceVolume(int Volume)
        { 
                float fNewVolume = (float)0.0;
                if (float.TryParse(Volume.ToString(), out fNewVolume))
                {
                    DefaultAudioEndPointDevice.AudioEndpointVolume.MasterVolumeLevelScalar = (fNewVolume / 100);
                }
        }
        public void MuteAllSessionsExceptThisProcessName(String ProcessName)
        {
            Process[] aProcesses = Process.GetProcessesByName(ProcessName);
            //for (int iSession = 0; iSession < this.SessionsDefaultAudioEndPointDevice.Count; iSession++)
            foreach(AudioSession ASession in AudioSessions)
            {
                if (ASession != null)
                {
                    int ProcessCount = aProcesses.Count(s => s.Id == (ASession.Session.GetProcessID));
                    // IEnumerable<int> ProcessIDs = from s in aProcesses where s.Id == (this.SessionsDefaultAudioEndPointDevice[iSession].GetProcessID) select s.Id;
                    if (ProcessCount >= 1)
                    {
                        System.Diagnostics.Debug.WriteLine("Process " + ASession.SessionName + " found in Audiosesions => not muting");
                    }
                    else
                    {
                        System.Diagnostics.Debug.WriteLine("Muting " + ASession.SessionName);
                        ASession.Session.SimpleAudioVolume.Mute = true;
                    }
                }
            }
            LoadAudioSessions();
        }
        public void UnMuteAllSessions()
        {
            foreach(AudioSession ASession in AudioSessions)
            {
                if (ASession != null)
                {
                    System.Diagnostics.Debug.WriteLine("Unmuting " + ASession.SessionName);
                    ASession.Session.SimpleAudioVolume.Mute = false;
                }
            }
            LoadAudioSessions();
        }
        public event EventHandler<EventArgs> OnAudioSessionReloaded;
        protected virtual void AudioSessionsReload(EventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("AUDIO SESSIONS RELAODED");
            EventHandler<EventArgs> handler = OnAudioSessionReloaded;
            if (handler != null)
            {
                handler(this, e);
            }
        }
        public void OnAllVolumeChanged(object sender, VolumeEventArgs e)
        {
            OnVolumeChangedOccured(e);
        }
        public void OnAllStateChanged(object sender, StateChangedEventArgs e)
        {
            //System.Diagnostics.Debug.WriteLine("A SESSION CHANGED HIS STATE:" + e.NewState);
            LoadAudioSessions();
            OnStateChangedOccured(e);
        }
        public void Dispose()
        {
            // http://blogs.msdn.com/b/jonathanswift/archive/2006/10/03/dynamically-calling-an-unmanaged-dll-from-.net-_2800_c_23002900_.aspx
            GC.SuppressFinalize(this);
        }
        ~Sound()
        {
            Dispose();
        }
        private static String GetNameByID(UInt32 iPID)
        {
            try
            {
                Process oProcess = Process.GetProcessById((int)iPID);
                return (oProcess.ProcessName);
            }
            catch (Exception e)
            {
                return (String.Empty);
            }
        }
        protected virtual void OnNewAudioSessionOccured(SessionCreatedEventArgs e)
        {
            EventHandler<SessionCreatedEventArgs> handler = OnNewAudioSession;
            if (handler != null)
            {
                handler(this, e);
            }
        }
        public event EventHandler<SessionCreatedEventArgs> OnNewAudioSession;
        public class SessionCreatedEventArgs : EventArgs
        {
            public SessionCreatedEventArgs(NAudio.CoreAudioApi.Interfaces.IAudioSessionControl Session)
            {
                this.Session = Session;
            }
            NAudio.CoreAudioApi.Interfaces.IAudioSessionControl Session { get; set; }
        }
        public event EventHandler<VolumeEventArgs> OnMasterVolumeChanged;
        protected virtual void OnMasterVolumeChangedOccured(VolumeEventArgs e)
        {
            EventHandler<VolumeEventArgs> handler = OnMasterVolumeChanged;
            if (handler != null)
            {
                handler(this, e);
            }
        }

        public event EventHandler<VolumeEventArgs> OnVolumeChanged;
        protected virtual void OnVolumeChangedOccured(VolumeEventArgs e)
        {
            EventHandler<VolumeEventArgs> handler = OnVolumeChanged;
            if (handler != null)
            {
                handler(this, e);
            }
        }
        protected virtual void OnStateChangedOccured(StateChangedEventArgs e)
        {
            EventHandler<StateChangedEventArgs> handler = OnStateChangedGlobal;
            if (handler != null)
            {
                handler(this, e);
            }
        }
        public event EventHandler<StateChangedEventArgs> OnStateChangedGlobal;
        public class StateChangedEventArgs : EventArgs
        {
            public StateChangedEventArgs(NAudio.CoreAudioApi.Interfaces.AudioSessionState NewState)
            {
                this.NewState = NewState;
            }
            public NAudio.CoreAudioApi.Interfaces.AudioSessionState NewState { get; set; }
        }
        public event EventHandler<DisconnectSessionEventArgs> OnSessionDisconnect;
        public class DisconnectSessionEventArgs : EventArgs
        {
            public DisconnectSessionEventArgs(NAudio.CoreAudioApi.Interfaces.AudioSessionDisconnectReason DisconnectReason)
            {
                this.DisconnectReason = DisconnectReason;
            }
            public NAudio.CoreAudioApi.Interfaces.AudioSessionDisconnectReason DisconnectReason { get; set; }
        }
        // public event EventHandler<VolumeEventArgs> OnVolumeChanged;
        public class VolumeEventArgs : EventArgs
        {
            public VolumeEventArgs(float volume, bool isMuted)
            {
                this.volume = volume;
                this.isMuted = isMuted;
                this.eventContext = eventContext;
            }
            public float volume { get; set; }
            public bool isMuted { get; set; }
            public Guid eventContext { get; set; }
        }
        internal class EventClient : NAudio.CoreAudioApi.Interfaces.IAudioSessionEventsHandler
        {
            public delegate void VolumeEventHandler(object sender, VolumeEventArgs e);
            public event VolumeEventHandler VolumeChanged;
            public delegate void DisconnectSessionEventHandler(object sender, DisconnectSessionEventArgs e);
            public event DisconnectSessionEventHandler SessionDisconnect;
            public delegate void StateChangedEventHandler(object sender, StateChangedEventArgs e);
            public event StateChangedEventHandler StateChanged;
            public void OnSessionDisconnected(NAudio.CoreAudioApi.Interfaces.AudioSessionDisconnectReason DisconnectReason) {
                // System.Diagnostics.Debug.WriteLine("SESSION DISCONNECTED. EVENT CLIENT");
                if (SessionDisconnect != null)
                {
                    DisconnectSessionEventArgs oDisConnectSessionEventArgs = new DisconnectSessionEventArgs(DisconnectReason);
                    SessionDisconnect(this, oDisConnectSessionEventArgs);
                }
            }
            public void OnDisplayNameChanged(string NewDisplayName) { }
            public void OnGroupingParamChanged(ref System.Guid NewGroupingParam) { System.Diagnostics.Debug.WriteLine("GROUP CHANGED"); }
            public void OnIconPathChanged(string NewIconPath) {}
            public void OnStateChanged(NAudio.CoreAudioApi.Interfaces.AudioSessionState NewState) {
                // System.Diagnostics.Debug.WriteLine("STATE CHANGED. EVENT CLIENT");
                if (StateChanged != null)
                {
                    StateChangedEventArgs oDisConnectSessionEventArgs = new StateChangedEventArgs(NewState);
                    StateChanged(this, oDisConnectSessionEventArgs);
                }
            }
            public void OnChannelVolumeChanged(uint ChannelCount, System.IntPtr NewChannelVolumeArray, uint ChangedChannel) { }
            //public int OnSimpleVolumeChanged( float volume, bool isMuted, ref Guid eventContext )
            public void OnVolumeChanged(float Volume, bool Muted)
            {
                // System.Diagnostics.Debug.WriteLine("VOLUME CHANGED. EVENT CLIENT");
                if (VolumeChanged != null)
                {
                    VolumeEventArgs oEventArgsVolume = new VolumeEventArgs(Volume, Muted);
                    VolumeChanged(this, oEventArgsVolume);
                }
            }
        }
    }
}
