﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using System.Text;
using System.Timers;
using System.Threading.Tasks;

namespace Spotify
{
    class Spotify
    {
        private const double DEFAULT_UPDATE_INTERVAL=50;
        private String SpotifyProcessName = "";
        private Timer SpotifyTimer;
        private String LastSongName = "";
        private Process SpotifyProcess;
        Regex oRegExRemoveInvalidPath;
        public class SpotifyProcesses 
        {
            public String ProcessName { get; set; }
            List<Process> SpotifyProcessList { get; set; }
            public SpotifyProcesses(String Processname) 
            {
                this.ProcessName = Processname;
                SpotifyProcessList = new List<Process>();
            }
            public void FindProcesses()
            {
                // CleanUp
                for(int iProcLoop=SpotifyProcessList.Count-1;iProcLoop>=0;iProcLoop--)
                {
                    if(SpotifyProcessList[iProcLoop]!=null)
                    {
                        SpotifyProcessList[iProcLoop].Exited-=OnProcessExit;
                        SpotifyProcessList[iProcLoop].Dispose();
                    }
                }
                SpotifyProcessList.Clear();
                SpotifyProcessList.AddRange(Process.GetProcessesByName(ProcessName));
                for (int iProcLoop = SpotifyProcessList.Count - 1; iProcLoop >= 0; iProcLoop--)
                {
                    if (SpotifyProcessList[iProcLoop] != null)
                    {
                        SpotifyProcessList[iProcLoop].Exited += OnProcessExit;
                    }
                }
            }
            public bool SpotifyRunning
            {
                get
                {
                    if (SpotifyProcessList.Count>=1)
                    {
                        // Process has exited?
                        if (SpotifyProcessList.FindAll(p => p.HasExited == true).Count >= 1)
                        {
                            FindProcesses();
                        }
                        else
                        {
                            // Refresh
                            foreach(Process Current in SpotifyProcessList)
                            {
                                Current.Refresh();
                            }
                        }
                    }
                    else
                    {
                        FindProcesses();
                    }
                    if (SpotifyProcessList.Count >= 1)
                        return true;
                    else
                        return false;
                }
            }
            public String GetSpotifySongTitle
            {
                get
                {
                    Process ProcessWithTitle=SpotifyProcessList.Find(t => String.IsNullOrEmpty((t.MainWindowTitle))==false);
                    if (ProcessWithTitle != null)
                        return ProcessWithTitle.MainWindowTitle;
                    else
                        return String.Empty;
                }
            }
            private void OnProcessExit(object sender, EventArgs e)
            {
                FindProcesses();
            }
            public void Dispose()
            {
                for (int iProcLoop = SpotifyProcessList.Count - 1; iProcLoop >= 0; iProcLoop--)
                {
                    if (SpotifyProcessList[iProcLoop] != null)
                    {
                        SpotifyProcessList[iProcLoop].Exited -= OnProcessExit;
                        SpotifyProcessList[iProcLoop].Dispose();
                    }
                }
                SpotifyProcessList.Clear();
            }
            ~SpotifyProcesses()
            {
                Dispose();
            }
        }

        public double UpdateInterval{
            get
            {
                return (SpotifyTimer.Interval);
            }
            set 
            {
                SpotifyTimer.Interval = value;
            }
        }
        public bool SpotifyIsRunning { get; set; }
        public String CurrentlyPlayed { get; set; }
        public String BandName { get; set; }
        public String Song { get; set; }
        public SpotifyProcesses AllSpotifyProcesses { get; set; }
        public Spotify(double UpdateInterval)
        {
            SpotifyTimer = new Timer { Interval = UpdateInterval};
            init();
        }
        public Spotify(double UpdateInterval, String SpodifyProcessName)
        {
            this.SpotifyProcessName = SpodifyProcessName;
            SpotifyTimer = new Timer { Interval = UpdateInterval };
            init();
        }
        private void init()
        {
            SpotifyTimer.Elapsed += OnTimerElapsed;
            SpotifyIsRunning=false;
            CurrentlyPlayed = "";
            LastSongName = "";
            BandName = "";
            Song = "";
            oRegExRemoveInvalidPath=new Regex(@"["+(new String(Path.GetInvalidFileNameChars()))+(new String(Path.GetInvalidPathChars()))+@"]");
            AllSpotifyProcesses = new SpotifyProcesses(this.SpotifyProcessName);
        }
        public void Stop()
        {
           SpotifyTimer.Stop();
           this.SpotifyIsRunning = false;
            OnSpotifyStop(EventArgs.Empty);
        }
        public void Start()
        {
            if (AllSpotifyProcesses.SpotifyRunning)
            {
                SpotifyTimer.Start();
            }
            else
            {
               this.SpotifyIsRunning = false;
               OnSpotifyStop(EventArgs.Empty);
            }
        }
        private void UpdateTitleInformation(String CurrentlyPlayed)
        {
            if(CurrentlyPlayed.Length>=4)
            {
                String[] aTitle=CurrentlyPlayed.Split(new char[] { '-' }, 2, System.StringSplitOptions.RemoveEmptyEntries);
                if (aTitle.Length >= 2)
                {
                    BandName = aTitle[0].Trim();
                    Song = aTitle[1].Trim();
                }
            }
            else
            {
                BandName = "";
                Song = "";
            }
        }
        private String RemoveInvalidPathChars(String InputString)
        {
            return oRegExRemoveInvalidPath.Replace(InputString, "");
        }
        private void OnTimerElapsed(object sender, EventArgs e)
        {
            //if (SpotifyProcess != null)
            if(AllSpotifyProcesses.SpotifyRunning)
            {

                    if (!this.SpotifyIsRunning)
                    {
                        OnSpotifyStart(EventArgs.Empty);
                        this.SpotifyIsRunning = true;
                        System.Diagnostics.Debug.WriteLine("Spotify up");
                    }
                    
                    // Songtile = WindowTitle of 1st Spotify Process
                    CurrentlyPlayed = RemoveInvalidPathChars(
                        // Band & Song title >= 4 Characters?
                            AllSpotifyProcesses.GetSpotifySongTitle.Length >= 4 ?
                        // Yes, set
                            AllSpotifyProcesses.GetSpotifySongTitle.Trim() :
                        // No, return empty string
                            String.Empty
                    );
                    // Title Changes since last poll?
                    if (!String.IsNullOrEmpty(CurrentlyPlayed))
                    {
                        if (CurrentlyPlayed != LastSongName)
                        {
                            // No Song is played
                            if (CurrentlyPlayed=="Spotify")
                            {
                                Stop();
                                return;
                            }
                            LastSongName = CurrentlyPlayed;
                            System.Diagnostics.Debug.WriteLine("New Song:" + CurrentlyPlayed);
                            UpdateTitleInformation(CurrentlyPlayed);
                            NewSongEventArgs oEvtArgs = new NewSongEventArgs();
                            oEvtArgs.Song = new Song.Song(this.BandName, this.Song);
                            oEvtArgs.Band = this.BandName;
                            oEvtArgs.Songtitle = this.Song;
                            oEvtArgs.FileName = CurrentlyPlayed;
                            OnNewSong(oEvtArgs);
                        }
                    }
                

            }
            else
            {
                if (this.SpotifyIsRunning)
                {
                    this.SpotifyIsRunning = false;
                    System.Diagnostics.Debug.WriteLine("Spotify down");
                    CurrentlyPlayed = "";
                    LastSongName = "";
                    BandName = "";
                    Song = "";
                }
                SpotifyTimer.Stop();
                OnSpotifyStop(EventArgs.Empty);
            }
        }
        public void Dispose() 
        {
            AllSpotifyProcesses.Dispose();
        }
        ~Spotify()
        {
            Dispose();
        }
        // https://msdn.microsoft.com/en-us/library/9aackb16%28v=vs.110%29.aspx
        protected virtual void OnNewSong(NewSongEventArgs e)
        {
            EventHandler<NewSongEventArgs> handler = NewSong;
            if (handler != null)
            {
                handler(this, e);
            }
        }
        public event EventHandler<NewSongEventArgs> NewSong;
        public class NewSongEventArgs : EventArgs
        {
            public Song.Song Song { get; set;}     
            public String Band { get; set; }
            public String Songtitle { get; set; }
            public String FileName { get; set; }
        }
        protected virtual void OnSpotifyStart(EventArgs e)
        {
            EventHandler handler = SpotifyStarted;
            if (handler != null)
            {
                handler(this, e);
            }
        }
        public event EventHandler SpotifyStarted;
        protected virtual void OnSpotifyStop(EventArgs e)
        {
            EventHandler handler = SpotifyStopped;
            if (handler != null)
            {
                handler(this, e);
            }
        }
        public event EventHandler SpotifyStopped;
    }
}
