﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Diagnostics;
using Spotify;
using Recorder;
using Lame;
using MP3;
/*
 * 
 * SpotifyRecorderNG
 * 
 * http://michlstechblog.info
 * 
 * License: No license, feel free to modify
 * 
 * Licenses of libraries and tools used by this project
 * Lame               => LGPL http://lame.sourceforge.net/license.txt
 * NAudio             => Microsoft Public License (Ms-PL) https://naudio.codeplex.com/license
 * WindowsAPICodePack => Microsoft Public License (Ms-PL) http://www.codeproject.com/Articles/109403/Windows-API-Code-Pack-v-Released
 * 
*/
namespace SpotifyRecorderNG
{
    public partial class SpotifyRecorderNGMain : Form
    {
        Spotify.Spotify oSpotify;
        Recorder.Recorder oRecorder;
        Sound.Sound oSound;
        private int LastSongID = 0;
        private String SpotifyProcessName = "";
        public System.Collections.Hashtable hSongs;
        private List<Lame.Lame.Bitrate> Bitrates;
        private int MinimumAudioVolumePercent = 0;
        public SpotifyRecorderNGMain()
        {
            InitializeComponent();
        }
        private void SpotifyRecorderNGMain_Load(object sender, EventArgs e)
        {
            SpotifyProcessName = (String)SpotifyRecorderNG.Default["SpotifyProcessName"];
            oSound = new Sound.Sound(SpotifyProcessName);
            oSound.OnVolumeChanged += OnVolumeChanged;
            oSound.OnMasterVolumeChanged += OnMasterVolumeChanged;
            oSound.OnAudioSessionReloaded += OnAudioSessionsReloaded;
            int Volume = oSound.SpodifyVolume;
            Volume = oSound.DefaultAudioDeviceVolume;
            labelDefaultAudioDevice.Text = oSound.DefaultAudioEndPointDevice.FriendlyName;
            trackBarMasterVolume.Value = oSound.DefaultAudioDeviceVolume;
            MinimumAudioVolumePercent = (int)SpotifyRecorderNG.Default["MinimumVolumeInPercent"];
            oSpotify = new Spotify.Spotify((double)SpotifyRecorderNG.Default["DefaultSpotifyPollIntervall"],SpotifyProcessName);
            oSpotify.NewSong += OnNewSong;
            oSpotify.SpotifyStopped += OnSpotifyStop;
            oSpotify.SpotifyStarted += OnSpotifyStart;

            audioSessionBindingSource.DataSource = oSound.AudioSessions;
            labelOutPutDirectory.Text = (String)SpotifyRecorderNG.Default["OutPutDirectory"];
            if (String.IsNullOrEmpty(labelOutPutDirectory.Text) || !System.IO.File.Exists(labelOutPutDirectory.Text))
            {
                labelOutPutDirectory.Text = Environment.GetFolderPath(Environment.SpecialFolder.MyMusic);
            }
            checkBoxDelete15secTracks.Checked=(bool) SpotifyRecorderNG.Default["Delete15SecTracks"];
            checkBoxDelete30secTracks.Checked = (bool)SpotifyRecorderNG.Default["Delete30SecTracks"];
            checkBoxAudioScrobbler.Checked = (bool)SpotifyRecorderNG.Default["AudioScrobbler"];
            checkBoxMute.Checked = (bool)SpotifyRecorderNG.Default["MuteAudioSessions"];
            Lame.Lame.Bitrate SelectedBitrate=new Lame.Lame.Bitrate((int)SpotifyRecorderNG.Default["SelectedBitrate"]);
            Bitrates = Lame.Lame.Bitrates((String)SpotifyRecorderNG.Default["LameBitrates"]);
            bitrateBindingSource.DataSource = Bitrates;
            comboBoxBitrate.Text =SelectedBitrate.DisplayName;
            toolStripStatusLabelVersion.Text = (System.Diagnostics.FileVersionInfo.GetVersionInfo((System.Reflection.Assembly.GetExecutingAssembly()).Location)).FileVersion.ToString();
            if (!(bool)SpotifyRecorderNG.Default["WarningRead"])
            {
                if (MessageBox.Show(((String)SpotifyRecorderNG.Default["WarningMessage"]).Replace(@"\n", "\n"), "Disclaimer", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    SpotifyRecorderNG.Default["WarningRead"] = true;
                    SpotifyRecorderNG.Default.Save();
                }
                else
                {
                    Application.Exit();
                }
            }
        }
        private void OnAudioSessionsReloaded(object sender, EventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("RELOAD MAIN");
            this.BeginInvoke((Action)(() => audioSessionBindingSource.DataSource = oSound.AudioSessions));
            this.BeginInvoke((Action)(() => audioSessionBindingSource.ResetBindings(false)));
        }


        private void OnVolumeChanged(object sender, Sound.Sound.VolumeEventArgs e)
        {
            this.BeginInvoke((Action)(() => audioSessionBindingSource.ResetBindings(false)));
        }
        private void OnMasterVolumeChanged(object sender, Sound.Sound.VolumeEventArgs e)
        {
            this.BeginInvoke((Action)(() => trackBarMasterVolume.Value=(int)(e.volume*100)));
        }
        private void OnNewAudioSession(object sender, Sound.Sound.SessionCreatedEventArgs e)
        {
            if (checkBoxMute.Checked)
            {
                oSound.MuteAllSessionsExceptThisProcessName(SpotifyProcessName);
            }            
        }
        private void OnSpotifyStop(object sender, EventArgs e)
        {
            toolStripStatusLabelSpotifyStatus.Text = "Spotify is not running";
            if (oRecorder != null)
            {
                if (oRecorder.IsRecording)
                {
                    oRecorder.StopRecording();
                    oRecorder.OnRecordingFinshed -= OnRecordingFinished;
                    oRecorder.Dispose();
                }
            }
            // Delegate to be Thread safe
            this.BeginInvoke((Action)(() => buttonStartRecording.Enabled = true));
            this.BeginInvoke((Action)(() => buttonStopRecording.Enabled = false));
            this.BeginInvoke((Action)(() => toolStripStatusLabelColor.BackColor = Color.Red));
            this.BeginInvoke((Action)(() => songBindingSource.ResetBindings(false))); ;
        }
        private void OnSpotifyStart(object sender, EventArgs e)
        {
            toolStripStatusLabelSpotifyStatus.Text = "Spotify is running";
            // Delegate to be Thread safe
            // Refresh DataGrid
            oSound.LoadAudioSessions();
            this.BeginInvoke((Action)(() => toolStripStatusLabelColor.BackColor = Color.LightGreen));
            if (oRecorder != null)
            {
                if (oRecorder.IsRecording)
                {
                    oRecorder.StopRecording();
                    oRecorder.OnRecordingFinshed -= OnRecordingFinished;
                    oRecorder.Dispose();
                }
            }
            this.BeginInvoke((Action)(() => buttonStopRecording.Enabled = true));
            this.BeginInvoke((Action)(() => buttonStartRecording.Enabled = false));
        }
        private void OnNewSong(object sender, Spotify.Spotify.NewSongEventArgs e)
        {
            if (hSongs == null)
                hSongs = new System.Collections.Hashtable();
             this.BeginInvoke((Action)(() => pictureBoxCover.Enabled = false));
            if ((!String.IsNullOrEmpty(e.Band)) && (!String.IsNullOrEmpty(e.Songtitle)))
            {
                e.Song.SongID = ++LastSongID;
                e.Song.WorkingDirectory = labelOutPutDirectory.Text;
                hSongs.Add(e.Song.SongID, e.Song);
                if (oRecorder != null)
                {
                    if (oRecorder.IsRecording)
                    {
                        oRecorder.StopRecording();
                        oRecorder.OnRecordingFinshed -= OnRecordingFinished;
                        oRecorder.Dispose();
                    }
                }
                if ((!System.IO.File.Exists(e.Song.WavFilePath)) && (!System.IO.File.Exists(e.Song.MP3FilePath)))
                {
                    this.BeginInvoke((Action)(() => labelCurrentlyPlayed.Text = String.Format("{0} - {1}", e.Song.Artist, e.Songtitle)));
                    oRecorder = new Recorder.Recorder(e.Song);
                    oRecorder.StartRecording();
                    oRecorder.OnRecordingFinshed += OnRecordingFinished;
                    if (checkBoxAudioScrobbler.Checked) 
                        Metadata.Metadata.AudioScrobbler.QueryAudioScrobbler(e.Song);
                    if (e.Song.AlbumImage != null)
                    {
                         this.BeginInvoke((Action)(() => pictureBoxCover.Image = e.Song.AlbumImage));
                         this.BeginInvoke((Action)(() => pictureBoxCover.Enabled = true));
                    }
                }
                else
                {
                    System.Diagnostics.Debug.WriteLine("Song already exists:" + e.Song.Artist+" - "+e.Song.Title+" at "+e.Song.WorkingDirectory);
                    e.Song.CoversationState = (int)Song.Song.CONVERSATION_STATE.FILE_EXISTS;
                }
            }
            this.BeginInvoke((Action)(() => songBindingSource.DataSource = hSongs.Values.Cast<Song.Song>().ToList()));
        }
        private void button2_Click(object sender, EventArgs e)
        {

            if (oSound.DefaultAudioDeviceVolume < (MinimumAudioVolumePercent - 1) && oSound.SpodifyVolume >= 0 && oSound.SpodifyVolume < (MinimumAudioVolumePercent - 1))
            {
                if (MessageBox.Show("Output Volume is currently " + oSound.DefaultAudioDeviceVolume + "% and Spodify Volume is at " + oSound.SpodifyVolume + "%.\n\r For a better dynamic of the output files it is neccessary to adjust it to greater then " + MinimumAudioVolumePercent+ "%.\n\r\n\rShould I do this for you?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    oSound.SetDefaultAudioDeviceVolume(MinimumAudioVolumePercent);
                    oSound.SetSpodifyVolume(MinimumAudioVolumePercent);
                }
            }
            else if (oSound.DefaultAudioDeviceVolume < (MinimumAudioVolumePercent - 1))
            {
                if (MessageBox.Show("Output Volume is currently " + oSound.DefaultAudioDeviceVolume + "%. For a better dynamic of the output files it is neccessary to adjust it to greater then " + MinimumAudioVolumePercent + "%.\n\r\n\rShould I do this for you?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    oSound.SetDefaultAudioDeviceVolume(MinimumAudioVolumePercent);
                }
            }
            else if (oSound.SpodifyVolume >= 0 && oSound.SpodifyVolume < (MinimumAudioVolumePercent - 1))
            {
                if (MessageBox.Show("Spodify Volume is currently " + oSound.SpodifyVolume + "%. For a better dynamic of the output files it is neccessary to adjust it to greater then "+MinimumAudioVolumePercent+"%.\n\r\n\rShould I do this for you?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    oSound.SetSpodifyVolume(MinimumAudioVolumePercent);
                }
            }
            oSpotify.Start();
            oSound.LoadAudioSessions();
            buttonChangeOutputDirectory.Enabled = false;
        }

        private void button3_Click(object sender, EventArgs e)
        {
            oSpotify.Stop();
            if (oRecorder != null)
            {
                oRecorder.StopRecording();
            }
            buttonChangeOutputDirectory.Enabled = true;
        }
        private void OnRecordingFinished(object sender, Recorder.Recorder.RecordingFinishedEventArgs e)
        {
            if (e.Song.CoversationState == (int)Song.Song.CONVERSATION_STATE.RECORDING_FINSIHED)
            {
                Lame.Lame.LameOptions oLameOptions = new Lame.Lame.LameOptions(e.Song, 192);
                oLameOptions.LameCustomParameters = (String)SpotifyRecorderNG.Default["CustomLameParameters"];
                Lame.Lame oLameMP3Converter = new Lame.Lame(new FileInfo(Application.ExecutablePath), oLameOptions);
                if (oLameMP3Converter.oSong.CoversationState != (int)Song.Song.CONVERSATION_STATE.CONVERT_ERROR)
                {
                    oLameMP3Converter.OnRecordingFinshed += OnLameConvertExit;
                    oLameMP3Converter.OnStateUpdate += OnLameStatusUpdate;
                    oLameMP3Converter.Convert2MP3();
                }
            }
            this.BeginInvoke((Action)(() => songBindingSource.ResetBindings(false)));
        }
        private void OnLameStatusUpdate(object sender, Lame.Lame.ConvertingStatsEventArgs e)
        {
            // Refresh DataGrid
            this.BeginInvoke((Action)(() => songBindingSource.ResetBindings(false)));
        }
        private void OnLameConvertExit(object sender, Lame.Lame.ConvertingFinishedEventArgs e)
        {
            if (e.DeleteSourceFile)
            {
                try
                {
                    System.IO.File.Delete(e.Inputfile);
                }
                catch (Exception eEx)
                {
                    System.Diagnostics.Debug.WriteLine("Cannot delete file " + e.Inputfile + " Error:" + eEx.Message);
                }
            }
            // AlbumImage
            if (!String.IsNullOrEmpty(e.Song.AlbumImagePath))
            {
                 try
                {
                    System.IO.File.Delete(e.Song.AlbumImagePath);
                }
                catch (Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine("Cannot delete Album Image File:"+ex.Message);
                }
            }
            MP3.MP3.MP3Details MP3Details = MP3.MP3.GetMP3Details(e.Outputfile);
            // 15 sec Advertising Files
            if (MP3Details.length==15 && checkBoxDelete15secTracks.Checked)
            {
                // Delete
                System.Diagnostics.Debug.WriteLine("Deleting Advertising tracks " + e.Outputfile);
                try
                {
                    e.Song.CoversationState = (int)Song.Song.CONVERSATION_STATE.DELETED;
                    System.IO.File.Delete(e.Outputfile);
                }
                catch (Exception eEx)
                {
                    System.Diagnostics.Debug.WriteLine("Cannot delete file " + e.Outputfile + " Error:" + eEx.Message);
                }
            }
            // 30 sec Advertising Files
            if (MP3Details.length == 30 && checkBoxDelete30secTracks.Checked)
            {
                // Delete
                System.Diagnostics.Debug.WriteLine("Deleting Advertising tracks " + e.Outputfile);
                try
                {
                    e.Song.CoversationState = (int)Song.Song.CONVERSATION_STATE.DELETED;
                    System.IO.File.Delete(e.Outputfile);
                }
                catch (Exception eEx)
                {
                    System.Diagnostics.Debug.WriteLine("Cannot delete file " + e.Outputfile + " Error:" + eEx.Message);
                }
            }

            // Refresh DataGrid
            this.BeginInvoke((Action)(() => songBindingSource.ResetBindings(false)));

        }
        private void SpotifyRecorderNGMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (oSpotify != null)
            {
                oSpotify.NewSong -= OnNewSong;
                oSpotify.SpotifyStopped -= OnSpotifyStop;
                oSpotify.SpotifyStarted -= OnSpotifyStart;
                oSpotify.Stop();
                oSpotify.Dispose();
            }
            if (oRecorder != null)
            {
                oRecorder.OnRecordingFinshed -= OnRecordingFinished;
                oRecorder.StopRecording();
                oRecorder.Dispose();
            }
            if (checkBoxMute.Checked)
            {
                oSound.UnMuteAllSessions();
            }   
        }

        private void checkBoxDelete30secTracks_CheckedChanged(object sender, EventArgs e)
        {
            SpotifyRecorderNG.Default["Delete30SecTracks"] = checkBoxDelete30secTracks.Checked;
            SpotifyRecorderNG.Default.Save();
        }

        private void checkBoxDelete15secTracks_CheckedChanged(object sender, EventArgs e)
        {
            SpotifyRecorderNG.Default["Delete15SecTracks"] = checkBoxDelete15secTracks.Checked;
            SpotifyRecorderNG.Default.Save();
        }

        private void buttonChangeOutputDirectory_Click(object sender, EventArgs e)
        {
            folderBrowserDialog.SelectedPath = labelOutPutDirectory.Text;
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                labelOutPutDirectory.Text = folderBrowserDialog.SelectedPath;
                SpotifyRecorderNG.Default["OutPutDirectory"] = labelOutPutDirectory.Text;
                SpotifyRecorderNG.Default.Save();
            }
        }

        private void checkBoxMute_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxMute.Checked)
            {
                oSound.MuteAllSessionsExceptThisProcessName(SpotifyProcessName);
            }
            else
            {
                oSound.UnMuteAllSessions();
            }
            SpotifyRecorderNG.Default["MuteAudioSessions"] = checkBoxMute.Checked;
            SpotifyRecorderNG.Default.Save();
        }

        private void comboBoxBitrate_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBoxBitrate.SelectedItem != null)
            {
                SpotifyRecorderNG.Default["SelectedBitrate"] = ((Lame.Lame.Bitrate)comboBoxBitrate.SelectedItem).Value;
                SpotifyRecorderNG.Default.Save();
            }
        }

        private void toolStripStatusLabelSpodifyEUA_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(((String)SpotifyRecorderNG.Default["WarningMessage"]).Replace(@"\n", "\n"), "Disclaimer", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
            {
                SpotifyRecorderNG.Default["WarningRead"] = true;
                SpotifyRecorderNG.Default.Save();
            }
            else
            {
                SpotifyRecorderNG.Default["WarningRead"] = false;
                SpotifyRecorderNG.Default.Save();
                Application.Exit();
            }
        }

        private void checkBoxAudioScrobbler_CheckedChanged(object sender, EventArgs e)
        {
            SpotifyRecorderNG.Default["AudioScrobbler"] = checkBoxAudioScrobbler.Checked;
            SpotifyRecorderNG.Default.Save();
        }
    }
}
