﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.ComponentModel;

namespace StringLoader
{
    public class StringLoader : IDisposable
    {
        [DllImport("kernel32", CharSet = CharSet.Auto)]
        static extern IntPtr LoadLibrary(string filename);
        [DllImport("kernel32", CharSet = CharSet.Auto)]
        static extern bool FreeLibrary(IntPtr lib);
        [DllImport("user32", CharSet = CharSet.Auto)]
        static extern int LoadString(IntPtr hInstance, int uID, out IntPtr buffer, int BufferMax);
        IntPtr plibPtr;
        public StringLoader(string sFileName)
        {
            String sDLL = Environment.ExpandEnvironmentVariables(sFileName);
                plibPtr = LoadLibrary(sDLL);
                if (plibPtr == IntPtr.Zero)
                    throw new Win32Exception();
        }
        public string Load(int id)
        {
            IntPtr resource;
            int length = LoadString(plibPtr, id, out resource, 0);
            if (length == 0)
                return null;
            else
                return Marshal.PtrToStringAuto(resource, length);
        }
        public void Dispose()
        {
            // http://blogs.msdn.com/b/jonathanswift/archive/2006/10/03/dynamically-calling-an-unmanaged-dll-from-.net-_2800_c_23002900_.aspx
            FreeLibrary(plibPtr);
            GC.SuppressFinalize(this);
        }
        ~StringLoader()
        {
            Dispose();
        }
    }
    // Loads a String from a DLL
    public class LoadStringFromDLL
    {
        public static String LoadStringFromDLLByPath(String StringPath)
        {
            if(StringPath.StartsWith(@"@"))
            {
                String[] aPathDetails=StringPath.Split(new char[] {','});
                String DllString = String.Empty;
                if(aPathDetails.Length==2)
                {
                    String sDLLPath = aPathDetails[0].Replace(@"@", "");
                    if (!System.IO.File.Exists(Environment.ExpandEnvironmentVariables(sDLLPath)))
                    {
                           return String.Empty;
                    }
                    StringLoader sl = new StringLoader(sDLLPath);
                    try
                    {
                            DllString = sl.Load(Convert.ToInt32(aPathDetails[1].Replace("-", "")));
                    }
                    catch (Exception e)
                    {
                         System.Diagnostics.Debug.WriteLine("Could not extract String from DLL Path " + StringPath+ ". Error:"+e.Message);
                         DllString = String.Empty;
                    }
                    sl.Dispose();
                    return DllString;
                }
                return String.Empty;
            }
            else{
                return StringPath;
            }
        }
    }
}
